import abjad
import abjadext.nauert


def test_Quantizer___call___01():
    milliseconds = [1500, 1500]
    q_events = abjadext.nauert.QEventSequence.from_millisecond_durations(
        milliseconds)
    quantizer = abjadext.nauert.Quantizer()
    result = quantizer(q_events)
    staff = abjad.Staff([result], lilypond_type='RhythmicStaff')
    score = abjad.Score([staff])
    assert format(score) == abjad.String.normalize(
        r'''
        \new Score
        <<
            \new RhythmicStaff
            {
                \new Voice
                {
                    {   % measure
                        \time 4/4
                        \tempo 4=60
                        c'4.
                        c'4.
                        r4
                    }   % measure
                }
            }
        >>
        '''
        ), format(score)


def test_Quantizer___call___02():
    milliseconds = [750, 750]
    q_events = abjadext.nauert.QEventSequence.from_millisecond_durations(
        milliseconds)
    quantizer = abjadext.nauert.Quantizer()
    optimizer = abjadext.nauert.MeasurewiseAttackPointOptimizer()
    result = quantizer(
        q_events,
        attack_point_optimizer=optimizer,
        )
    staff = abjad.Staff([result], lilypond_type='RhythmicStaff')
    score = abjad.Score([staff])
    assert format(score) == abjad.String.normalize(
        r'''
        \new Score
        <<
            \new RhythmicStaff
            {
                \new Voice
                {
                    {   % measure
                        \time 4/4
                        \tempo 4=60
                        c'8.
                        c'16
                        ~
                        c'8
                        r8
                        r2
                    }   % measure
                }
            }
        >>
        '''
        ), format(score)


def test_Quantizer___call___03():

    milliseconds = [1500, -1000, 1000, 1000, -1000, 1000, -1000, 500]
    sequence = abjadext.nauert.QEventSequence.from_millisecond_durations(milliseconds)
    attack_point_optimizer = abjadext.nauert.NullAttackPointOptimizer()
    quantizer = abjadext.nauert.Quantizer()

    result = quantizer(sequence, attack_point_optimizer=attack_point_optimizer)

    assert isinstance(result, abjad.Voice)
    assert abjad.inspect(result).get_duration() == 2

    score = abjad.Score([abjad.Staff([result])])

    assert format(score) == abjad.String.normalize(
        r'''
        \new Score
        <<
            \new Staff
            {
                \new Voice
                {
                    {   % measure
                        \time 4/4
                        \tempo 4=60
                        c'4
                        ~
                        c'8
                        r8
                        r8
                        c'8
                        ~
                        c'8
                        c'8
                        ~
                    }   % measure
                    {   % measure
                        c'8
                        r8
                        r8
                        c'8
                        ~
                        c'8
                        r8
                        r8
                        c'8
                    }   % measure
                }
            }
        >>
        '''
        ), format(score)


def test_Quantizer___call___04():

    milliseconds = [250, 1000, 1000, 1000, 750]
    sequence = abjadext.nauert.QEventSequence.from_millisecond_durations(
        milliseconds)
    attack_point_optimizer = abjadext.nauert.NullAttackPointOptimizer()
    quantizer = abjadext.nauert.Quantizer()
    result = quantizer(sequence, attack_point_optimizer=attack_point_optimizer)
    staff = abjad.Staff([result], lilypond_type='RhythmicStaff')
    score = abjad.Score([staff])

    assert format(score) == abjad.String.normalize(
        r'''
        \new Score
        <<
            \new RhythmicStaff
            {
                \new Voice
                {
                    {   % measure
                        \time 4/4
                        \tempo 4=60
                        c'16
                        c'16
                        ~
                        c'8
                        ~
                        c'16
                        c'16
                        ~
                        c'8
                        ~
                        c'16
                        c'16
                        ~
                        c'8
                        ~
                        c'16
                        c'16
                        ~
                        c'8
                    }   % measure
                }
            }
        >>
        '''
        ), format(score)


def test_Quantizer___call___05():

    q_schema = abjadext.nauert.BeatwiseQSchema(
        {'search_tree': abjadext.nauert.UnweightedSearchTree({2: None})},
        {'search_tree': abjadext.nauert.UnweightedSearchTree({3: None})},
        {'search_tree': abjadext.nauert.UnweightedSearchTree({5: None})},
        {'search_tree': abjadext.nauert.UnweightedSearchTree({7: None})},
        )
    milliseconds = [250, 250, 250, 250] * 4
    q_events = abjadext.nauert.QEventSequence.from_millisecond_durations(
        milliseconds)
    attack_point_optimizer = abjadext.nauert.NullAttackPointOptimizer()
    quantizer = abjadext.nauert.Quantizer()

    result = quantizer(
        q_events,
        q_schema=q_schema,
        attack_point_optimizer=attack_point_optimizer,
        )
    staff = abjad.Staff([result], lilypond_type='RhythmicStaff')
    score = abjad.Score([staff])

    assert format(score) == abjad.String.normalize(
        r'''
        \new Score
        <<
            \new RhythmicStaff
            {
                \new Voice
                {
                    \grace {
                        c'16
                    }
                    \tempo 4=60
                    c'8
                    \grace {
                        c'16
                    }
                    c'8
                    \times 2/3 {
                        c'8
                        \grace {
                            c'16
                        }
                        c'8
                        c'8
                    }
                    \times 4/5 {
                        c'16
                        c'16
                        c'16
                        ~
                        c'16
                        c'16
                    }
                    \times 4/7 {
                        c'16
                        ~
                        c'16
                        c'16
                        c'16
                        ~
                        c'16
                        c'16
                        ~
                        c'16
                    }
                }
            }
        >>
        '''
        ), format(score)
